/*!
 * vue-interactjs v0.1.4 
 * (c) 2020 yoroshikudozo
 * Released under the MIT License.
 */
import Vue from 'vue';
import interact from 'interactjs';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

var dragEvents = ["dragstart", "dragmove", "draginertiastart", "dragend"];
var resizeEvents = ["resizestart", "resizemove", "resizeinertiastart", "resizeend"];
var gestureEvents = ["gesturestart", "gesturemove", "gestureend"];
var pointerEvents = ["down", "move", "up", "cancel", "tap", "doubletap", "hold"];
var allEvents = dragEvents.concat(resizeEvents).concat(gestureEvents).concat(pointerEvents);

var bindInteractEvents = function bindInteractEvents(interact, emit) {
  allEvents.forEach(function (eventName) {
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    interact.on(eventName, function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      emit.apply(void 0, [eventName].concat(args));
    });
  });
};

var InteractInstance = "InteractInstance";
function getInteractComponent(interact) {
  return Vue.extend({
    name: "Interact",
    props: {
      draggable: {
        type: Boolean,
        required: false
      },
      dragOption: {
        type: Object,
        default: function _default() {
          return {};
        }
      },
      resizable: {
        type: Boolean,
        required: false
      },
      resizeOption: {
        type: Object,
        default: function _default() {
          return {};
        }
      },
      droppable: {
        type: Boolean,
        required: false
      },
      dropOption: {
        type: Object,
        default: function _default() {
          return {};
        }
      },
      gesturable: {
        type: Boolean,
        required: false
      },
      gestureOption: {
        type: Object,
        default: function _default() {
          return {};
        }
      },
      tag: {
        type: String,
        default: "div"
      }
    },
    data: function data() {
      return _defineProperty({}, InteractInstance, null);
    },
    computed: {
      interactInstance: {
        cache: false,
        set: function set(interact) {
          this[InteractInstance] = interact;
        },
        get: function get() {
          return this[InteractInstance];
        }
      }
    },
    mounted: function mounted() {
      if (!this.interactInstance) this.initInteract();
      if (this.draggable) this.initDrag();
      if (this.resizable) this.initResize();
      if (this.droppable) this.initDrop();
      if (this.gesturable) this.initGesture();
    },
    methods: {
      initInteract: function initInteract() {
        this.interactInstance = interact(this.$el);
        bindInteractEvents(this.interactInstance, this.$emit.bind(this));
        this.$emit("ready", this.interactInstance);
      },
      initDrag: function initDrag() {
        var _a;

        (_a = this.interactInstance) === null || _a === void 0 ? void 0 : _a.draggable(this.dragOption);
      },
      initResize: function initResize() {
        var _a;

        (_a = this.interactInstance) === null || _a === void 0 ? void 0 : _a.resizable(this.resizeOption);
      },
      initDrop: function initDrop() {
        var _a;

        (_a = this.interactInstance) === null || _a === void 0 ? void 0 : _a.dropzone(this.dropOption);
      },
      initGesture: function initGesture() {
        var _a;

        (_a = this.interactInstance) === null || _a === void 0 ? void 0 : _a.gesturable(this.gestureOption);
      }
    },
    render: function render(createElement) {
      return createElement(this.tag, {
        staticClass: "interact"
      }, this.$slots.default);
    }
  });
}

var version = "0.1.4";

var install = function install(Vue) {
  var InteractComponent = getInteractComponent(interact);
  Vue.component("Interact", InteractComponent);
};

var plugin = {
  install: install,
  version: version
};

if (typeof window !== "undefined" && window.Vue) {
  window.Vue.use(plugin);
}

export default plugin;
